#include "header.h"

// Calcule le nombre de possibilités sur une case
// Entrée : - matrice de cases sudoku
//          - case pour laquelle il faut faire le calcul
// Pré-condition : case non nulle
// Sortie : - entier nbPos contenant le nombre de possibilités
// Post-condition : nbPos positif ou nul
// Testée & validée
int nombrePossibilites(cases** sudoku, cases cellule, int n) {
  // Variable renvoyee
  int nbPoss = 0;

  for (int k = 1; k <= n * n; k++) {
    cellule.valeur = k;
    if (estValide(sudoku, cellule, n)) {
      nbPoss++;
    }
  }
  
  cellule.valeur = 0;
  return nbPoss;
}

// Verification de la validité d'une cellule
// Entrée : - Sudoku
//          - Cellule à vérifier
// Sortie : - Booléen : - true si la cellule est valide
//                      - false sinon
// Pré-condition : la cellule fait partie de la grille de Sudoku
// Non testée, ne marche pas
bool estValide(cases** sudoku, cases cellule, int n) {
  // Déclaration des variables

  int x = cellule.coordonnees.x;
  int y = cellule.coordonnees.y;

  // Vérification sur la ligne
  for (int j = 0; j < n * n; j++) {
    if (sudoku[x][j].valeur == cellule.valeur) {
      return false;
    }
  }

  // Vérification sur la colonne
  for (int i = 0; i < n * n; i++) {
    if (sudoku[i][y].valeur == cellule.valeur) {
      return false;
    }
  }

  // Vérification dans le grand carré de taille n :

  // Retrouve l'abscice du grand carré
  int entierex = x / n;
  // Retrouve l'ordonnée du grand carré
  int entierey = y / n;

  // Boucle de parcours du grand carre
  for (int i = 0; i < n; i++) {
    for (int j = 0; j < n; j++)
      if (sudoku[i + entierex * n][j + entierey * n].valeur == cellule.valeur) {
        return false;
      }
  }

  return true;
}

// Remplit les possibilitées de chaque cellule
// Entrée : - Sudoku
//          - Taille
// Pré-condition : Le Sudoku est rempli
//                Taille non nulle
// Post-condition : Le Sudoku est rempli avec le nombre de possibilités
// Testée & validée
void remplirPoss(cases** grille, int n) {
  for (int i = 0; i < n * n; i++) {
    for (int j = 0; j < n * n; j++) {
      if (grille[i][j].valeur == 0) // C'est une case à remplir
      {
        grille[i][j].possibilites = nombrePossibilites(grille, grille[i][j], n);
      }
    }
  }
}

// Affiche le sudoku
// Entrée :
// Pré-condition : n < 10
// Sortie :
// Post-condition :
void afficheSudoku(cases** sudoku, int n) {
  for (int i = 0; i <= n * n; i++) {
    if (i == 0) {
      // printf("╭─");
      // for(int j = 0; j < n * n; j++) {
      //  if(j % n == 0 && j != 0) printf("──┬──");
      //  else printf("─");
      //}
      // printf("──╮\n");
      printf("|");
    } else if (i == n * n) {
      // printf("╰─");
      // for(int j = 0; j < n * n; j++) {
      //  if(j % n == 0 && j != 0) printf("──┴──");
      //  else printf("─");
      //}
      // printf("──╯\n");
      break;
    } else if (i % n == 0) {
      for (int j = 0; j < n * n; j++) {
        if (j % n == 0) {
          if (j == 0)
            printf("|");
          else
            printf("--|");
        } else
          printf("--");
      }
      printf("--|\n|");
    } else {
      printf("|");
    }

    for (int j = 0; j < n * n; j++) {
      if (j % n == 0 && j != 0) {
        printf("|");
      }

      int num = sudoku[i][j].valeur;

      if (num == 0)
        printf("  ");
      else if (num < 10)
        printf(" %d", num);
      else
        printf("%d", num);
    }

    printf("|\n");
  }

  printf("\n\n");
}

// Résout le sudoku
// Entrée : - matrice de cases grille : sudoku
//			- liste des cases à traiter
// 			- entier n : taille du sudoku
// Pré-conditions : - sudoku non nul
// 					- taille non nulle
// Sortie : - sudoku complété
// Post-condition : - sudoku valide
void remplitSudoku(cases** grille, listeChainee liste, int n) {
  noeud* noeudActuel = (noeud*)malloc(sizeof(noeud));
  assert(noeudActuel != NULL);
  noeudActuel = liste.premierElement;

  // pour chacune des cases à remplir
  while (noeudActuel != NULL) {
    // on tente chacune des possibilités pour remplir cette case
    for (int i = noeudActuel->carre.valeur + 1; i <= n * n; i++) {
      noeudActuel->carre.valeur = i;
      if (estValide(grille, noeudActuel->carre, n)) {
        int x = noeudActuel->carre.coordonnees.x;
        int y = noeudActuel->carre.coordonnees.y;
        grille[x][y].valeur = i;
        noeudActuel = noeudActuel->suivant;
        break;
      }
    }
    if (noeudActuel != NULL && noeudActuel != liste.premierElement &&
        noeudActuel->carre.valeur != 0) {
      grille[noeudActuel->carre.coordonnees.x][noeudActuel->carre.coordonnees.y].valeur = 0;
      noeudActuel->carre.valeur = 0;
      noeudActuel = noeudActuel->precedent;
    }
  }
}