#include "header.h"

// tri bulle
noeud* tri(listeChainee *liste) {
  noeud* racine = liste->premierElement;
  int longueur = liste->longueur;

  // parent de la racine pour pouvoir stocker le 1er nœud de la liste chaînée
  noeud parentRacine;
  parentRacine.suivant = racine;
  parentRacine.carre.possibilites = -1;

  // initialisation des pointeurs sur le nœud précédent, actif et suivant
  // on a précédent → actif → suivant
  noeud* noeudPrecedent = &parentRacine;
  noeud* noeudActif = racine;
  noeud* noeudSuivant = racine->suivant;

  noeud* temp; // nœud temporaire pour l'échange

  for (int i = 1; i < longueur; i++) {
    // on ré-initialise les nœuds précédent, actif et suivant
    noeudPrecedent = &parentRacine;
    noeudActif = noeudPrecedent->suivant;
    noeudSuivant = noeudActif->suivant;

    for (int j = 1; j < longueur; j++) {
      // s'il ne sont pas dans le bon ordre, on les échange
      if (noeudActif->carre.possibilites > noeudSuivant->carre.possibilites) {
        // on échange le nœud actif et le nœud suivant
        echange(noeudActif, noeudSuivant, noeudPrecedent);
        temp = noeudActif;
        noeudActif = noeudSuivant;
        noeudSuivant = temp;
      }

      // on avance d'un nœud pour précédent, actif et suivant
      noeudPrecedent = noeudPrecedent->suivant;
      noeudActif = noeudActif->suivant;
      noeudSuivant = noeudSuivant->suivant;
    }
  }

  liste->premierElement = parentRacine.suivant;
  return parentRacine.suivant;
}

// l'ordre des nœuds est parentA → a → b
void echange(noeud* a, noeud* b, noeud* parentA) {
  // l'ordre des nœuds est parentA → a → b → suite
  noeud* suite = b->suivant;

  parentA->suivant = b; // parentA → b
  b->suivant = a;       // parentA → b → a
  a->suivant = suite;   // parentA → b → a → suite
  
  b->precedent = parentA;
  a->precedent = b;
  
  if (suite != NULL)
    suite->precedent = a;
}

// Crée un nouveau noeud
// Entrée : - cellule du sudoku
// Pré-condition : cellule non nulle
// Sortie : - Nouveau noeud avec un champ carre qui est la cellule entrée et un
// champ suivant a NULL Post-condition : Un noeud bien initialisé Testée &
// validée
noeud* creeNoeud(cases cellule) {
  noeud* nouveauNoeud = (noeud*)malloc(sizeof(noeud));
  assert(nouveauNoeud != NULL);
  nouveauNoeud->carre = cellule;
  nouveauNoeud->suivant = NULL;
  return nouveauNoeud;
}

// Replit la liste chainée avec les différentes cellules encore vides et
// retroune sa longueur
// Fonction récursive
// Entrée : - La liste chaînée
//          - La grille du sudoku
//          - La ligne sélectionée
//          - La colonne sélectionée
//          - La taille du sudoku
// Pré-condition : Longueur à 1
//                 Premier noeud de la liste chaînée tel qu'il soit le premier
//                 noeud du sudoku de valeur 0 Ligne à la valeur de
//                 premierNoeud.coordonee.y Colonne à la valeur de
//                 premierNoeud.coordonee.x
// Sortie : - Longueur de la liste chaînée
// Post-condition : Longueur non nulle
// Testée & validée (bah...non)
void remplirListe(listeChainee* liste, noeud* actuel, cases** grille, int ligne, int colonne, int n) {
  // case vide, on crée un nouvel élément
  if (grille[ligne][colonne].valeur == 0) {
    noeud *nouveauNoeud = creeNoeud(grille[ligne][colonne]);
    actuel->suivant = nouveauNoeud;
    nouveauNoeud->precedent = actuel;
    actuel = nouveauNoeud;
    liste->longueur++;
  }

  if (ligne == (n * n - 1) && colonne == (n * n - 1)) {
    // on ne fait rien
  } else if (colonne == (n * n - 1)) {
    remplirListe(liste, actuel, grille, ligne + 1, 0, n);
  } else {
    remplirListe(liste, actuel, grille, ligne, colonne + 1, n);
  }
}